function [Iplus,Iminus,w,time,numberiteration] = find_sign_pattern(A,sparsity)
% Function find_sign_pattern(A,sparsity)
% delivers a Recoverable Support (I,s) of a matrix A with size sparsity
% with I = union(Iplus,Iminus) and s = A(:,I)'*w.
%
% Input:    A               : Matrix for desired Recoverable Support
%           sparsity        : Size of desired Recoverable Support
%
% Output:   Iplus           : Index set for positive entries of s 
%           Iminus          : Index set for negative entries of s
%           w               : Dual Certificate
%           time            : Duration of time for this method
%           numberiterations: Number of iterations
% 
% Based on 'Computing and Analyzing Recoverable Supports for Sparse Reconstruction'
% by C. Kruschel and D. A. Lorenz
%
% -------
% authors: Christian Kruschel, Institute for Analysis and Algebra, TU Braunschweig
%          Dirk A. Lorenz, Institute for Analysis and Algebra, TU Braunschweig
% contact: c.kruschel [at] tu-braunschweig.de

tic;                                        % Start Measuring Time
numberiteration = 0;                        % Initialize a Performance Indicator

if sparsity > rank(A)
    error('Sparsity must be less than the rank of A');
end

n = size(A,2);                              % Set Number of Columns of Matrix
tmp = sum(abs(A.^2));                       % Compute Norms of Columns
[NormA,NormAIndex] = max(tmp);              % Identify Column with largeste Norm
AI = A(:,NormAIndex);                       % Set Column with largeste Norm
w = 1/NormA*AI;                             % Compute Dual Certificate for Recoverable Support (RS) with Size One
I = NormAIndex; s = A'*w;                   % Compute RS (I,s)
Ic = [1:NormAIndex-1,NormAIndex+1:n];       % Compute Complement of I

[NormA2,NormAIndex] = max(tmp(Ic));         % Check whether Matrix is Suitable
if abs(NormA - NormA2) < 1e-6 && norm(A(:,NormAIndex + 1) - AI) < 1e-6
    error('Not A Suitable Matrix: Matrix has at least two maximal columns with the same norm');
end

LengthI = 1;                                % Set Size of (I,s)
RankAI = 1;                                 % Set Rank of A(:,I)
[Q,R] = qr(AI);                             % Compute QR decomposition of AI

flag = 0;                                   % Initialize Counting Number of Used Bases Elements
while LengthI < sparsity                    % Start Interation
    
    if size(Q,2) > flag
        y = Q(:,end-flag);                  % Choose Direction
    else
        break;
    end
    z = A'*y; z(I) = 0;                    
                                            % Choose Direction Step
    Lambda = [(1-s(Ic))./z(Ic); -(1+s(Ic))./z(Ic)];
    LambdaCount = 0;
    while LambdaCount < n - LengthI
        numberiteration = numberiteration + 1;   % Count Iterations
        
        [~,LambdaMinIndex] = min(abs(Lambda));
        ATw_tmp = s + Lambda(LambdaMinIndex)*z;
        
        I_tmp = find(abs(abs(ATw_tmp) - 1) < 1e-6);
        LengthI_tmp = length(I_tmp);
        
        if LengthI_tmp > sparsity
            Lambda = Lambda([1:LambdaMinIndex-1,LambdaMinIndex+1:end]);
            
        else
            IdiffI_tmp = setdiff(I_tmp,I);          % Prepare for QR Rank One Update
            IdiffI_tmpLength = length(IdiffI_tmp);  % Determine Number of New Inserted Columns
            Q_tmp = Q; R_tmp = R;

            for p = 1:IdiffI_tmpLength              % Perform QR Rank One Update
                tmp = find(I < IdiffI_tmp(p));
                if isempty(tmp)
                    [Q_tmp,R_tmp] = qrinsert(Q_tmp,R_tmp,1,A(:,IdiffI_tmp(p)),'col');
                else
                    [Q_tmp,R_tmp] = qrinsert(Q_tmp,R_tmp,tmp(end)+1,A(:,IdiffI_tmp(p)),'col');
                end
            end

            RankAI_tmp = RankAI;                    % Determine Rank of A(:,I_tmp)
            while RankAI_tmp + 1 <= size(R_tmp,2)
                if abs(R_tmp(RankAI_tmp,RankAI_tmp)) > 1e-10
                    RankAI_tmp = RankAI_tmp + 1;
                else
                    break;
                end
            end
            
            if RankAI_tmp == LengthI_tmp           % If A(:,I_tmp) has full rank go on
                break;
            else                                   % else Choose a Different Direction Step
                Lambda = Lambda([1:LambdaMinIndex-1,LambdaMinIndex+1:end]);
            end
        end
        LambdaCount = LambdaCount + 1;
    end
    
    if LambdaCount == n - LengthI
        flag = flag + 1;
    else
        s = ATw_tmp;
        w = w + Lambda(LambdaMinIndex)*y;
        I = I_tmp;
        Ic = setdiff(1:n,I);
        Q = Q_tmp;
        R = R_tmp;
        LengthI = LengthI_tmp;
        RankAI = RankAI_tmp;
        
        flag = 0;
    end
end

Iplus = find(abs(A'*w - 1) < 1e-6);
Iminus = find(abs(A'*w + 1) < 1e-6);
time= toc;

if length(union(Iplus,Iminus)) - sparsity ~= 0
    disp(['new:An index set with sparity ',num2str(sparsity),' is not detectable.']);
  %  disp('Maybe use given output as an alternative');
end